package jsaiz.jython.bug2455;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;

import org.junit.After;
import org.junit.Test;
import org.python.core.PyJavaPackage;
import org.python.core.PyModule;
import org.python.util.PythonInterpreter;

import static org.junit.Assert.*;

/**
 * Test for the Jython bug 2455.
 * @author jsaiz
 */
public class TestBug2455 {

    private static final String NEW_LINE = System.getProperty("line.separator");

    private final PythonInterpreter interpreter = new PythonInterpreter();
    private final String example1 = "example1";
    private final String example2 = "example2";

    @After
    public void tearDown() {
        for (String dirName : new String[] { example1, example2 }) {
            File directory = new File(dirName);
            File[] files = directory.listFiles();
            for (File file : files) {
                file.delete();
            }
            directory.delete();
        }
        interpreter.close();
    }

    @Test(timeout = 60000)
    public void testJavaModule() throws IOException, InterruptedException {

        // Create Java class in example1 and __init__.py in example2
        createJavaFile(example1, "SomeClass");
        createInitFile(example2);

        // Create an interpreter and import the example packages
        interpreter.exec("import sys");
        interpreter.exec("sys.path.append('" + System.getProperty("user.dir") + "')");
        interpreter.exec("import " + example1);
        interpreter.exec("import " + example2);
        assertTrue(interpreter.eval(example1) instanceof PyJavaPackage);
        assertTrue(interpreter.eval(example2) instanceof PyModule);

        // Now add a Java class to example2 (after importing; otherwise example2 might be loaded as a PyJavaPackage)
        createJavaFile(example2, "OtherClass");

        // Both classes should be found
        evaluate(example1 + ".SomeClass");
        evaluate(example2 + ".OtherClass"); // works with 2.5.2 and the patch for 2.7.1, fails with 2.7.0
    }

    private void createJavaFile(String packageName, String className) throws IOException, InterruptedException {
        String javaCode = "package " + packageName + ";" + NEW_LINE + "public class " + className + " {}" + NEW_LINE;
        File javaFile = new File(packageName, className + ".java");
        createFile(javaFile, javaCode);
        assertEquals(0, Runtime.getRuntime().exec("javac " + javaFile.getPath()).waitFor());
    }

    private void createInitFile(String directory) throws IOException {
        File jythonFile = new File(directory, "__init__.py");
        createFile(jythonFile, "print 'within __init__.py'");
    }

    private void createFile(File file, String text) throws IOException {
        file.getParentFile().mkdirs();
        try (FileWriter writer = new FileWriter(file)) {
            writer.append(text);
            writer.flush();
        }
    }

    private void evaluate(String className) {
        assertEquals("<type '" + className + "'>", interpreter.eval(className).toString());
    }
}
